@@ -23,6 +23,7 @@
 #include <string.h>
 #include <strings.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "nd-notification.h"
 #include "nd-bubble.h"
@@ -48,7 +49,6 @@ struct NdBubblePrivate
         NdNotification *notification;
 
         GtkWidget      *main_hbox;
-        GtkWidget      *iconbox;
         GtkWidget      *icon;
         GtkWidget      *content_hbox;
         GtkWidget      *summary_label;
@@ -73,8 +73,6 @@ struct NdBubblePrivate
         guint           timeout_id;
 };
 
-static void     nd_bubble_class_init  (NdBubbleClass *klass);
-static void     nd_bubble_init        (NdBubble      *bubble);
 static void     nd_bubble_finalize    (GObject       *object);
 static void     on_notification_changed (NdNotification *notification,
                                          NdBubble       *bubble);
@@ -189,16 +187,33 @@ draw_round_rect (cairo_t *cr,
                    270.0f * G_PI / 180.0f);
 }
 
+static void
+get_background_color (GtkStyleContext *context,
+                      GtkStateFlags    state,
+                      GdkRGBA         *color)
+{
+        GdkRGBA *c;
+
+        g_return_if_fail (color != NULL);
+        g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
+
+        gtk_style_context_get (context, state,
+                               "background-color", &c,
+                               NULL);
+
+        *color = *c;
+        gdk_rgba_free (c);
+}
+
 static void
 paint_bubble (NdBubble *bubble,
               cairo_t  *cr)
 {
-        GdkColor         color;
-        double           r, g, b;
+        GtkStyleContext *context;
+        GdkRGBA          rgba;
         cairo_t         *cr2;
         cairo_surface_t *surface;
         cairo_region_t  *region;
-        GtkStyle        *style;
         GtkAllocation    allocation;
 
         gtk_widget_get_allocation (GTK_WIDGET (bubble), &allocation);
@@ -226,19 +241,16 @@ paint_bubble (NdBubble *bubble,
                          allocation.width - 2,
                          allocation.height - 2);
 
-        style = gtk_widget_get_style (GTK_WIDGET (bubble));
-        color = style->bg [GTK_STATE_NORMAL];
-        r = (float)color.red / 65535.0;
-        g = (float)color.green / 65535.0;
-        b = (float)color.blue / 65535.0;
-        cairo_set_source_rgba (cr2, r, g, b, BACKGROUND_ALPHA);
+        context = gtk_widget_get_style_context (GTK_WIDGET (bubble));
+
+        get_background_color (context, GTK_STATE_FLAG_NORMAL, &rgba);
+        cairo_set_source_rgba (cr2, rgba.red, rgba.green, rgba.blue,
+                               BACKGROUND_ALPHA);
         cairo_fill_preserve (cr2);
 
-        color = style->text_aa [GTK_STATE_NORMAL];
-        r = (float) color.red / 65535.0;
-        g = (float) color.green / 65535.0;
-        b = (float) color.blue / 65535.0;
-        cairo_set_source_rgba (cr2, r, g, b, BACKGROUND_ALPHA / 2);
+        gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &rgba);
+        cairo_set_source_rgba (cr2, rgba.red, rgba.green, rgba.blue,
+                               BACKGROUND_ALPHA / 2);
         cairo_set_line_width (cr2, 2);
         cairo_stroke (cr2);
 
@@ -357,8 +369,8 @@ nd_bubble_get_preferred_width (GtkWidget *widget,
 }
 
 static gboolean
-nd_bubble_motion_notify_event (GtkWidget        *widget,
-                               GdkEventCrossing *event)
+nd_bubble_motion_notify_event (GtkWidget      *widget,
+                               GdkEventMotion *event)
 {
         NdBubble *bubble = ND_BUBBLE (widget);
 
@@ -436,7 +448,6 @@ nd_bubble_init (NdBubble *bubble)
         GtkWidget   *vbox;
         GtkWidget   *close_button;
         GtkWidget   *image;
-        GtkWidget   *alignment;
         AtkObject   *atkobj;
         GdkScreen   *screen;
         GdkVisual   *visual;
@@ -469,19 +480,19 @@ nd_bubble_init (NdBubble *bubble)
                             bubble->priv->main_hbox,
                             FALSE, FALSE, 0);
 
-        /* First row (icon, vbox, close) */
-        bubble->priv->iconbox = gtk_alignment_new (0.5, 0, 0, 0);
-        gtk_widget_show (bubble->priv->iconbox);
-        gtk_alignment_set_padding (GTK_ALIGNMENT (bubble->priv->iconbox),
-                                   5, 0, 0, 0);
-        gtk_box_pack_start (GTK_BOX (bubble->priv->main_hbox),
-                            bubble->priv->iconbox,
-                            FALSE, FALSE, 0);
-        gtk_widget_set_size_request (bubble->priv->iconbox, BODY_X_OFFSET, -1);
+        /* Add icon */
 
         bubble->priv->icon = gtk_image_new ();
+        gtk_widget_set_valign (bubble->priv->icon, GTK_ALIGN_START);
+        gtk_widget_set_margin_top (bubble->priv->icon, 5);
+        gtk_widget_set_size_request (bubble->priv->icon, BODY_X_OFFSET, -1);
         gtk_widget_show (bubble->priv->icon);
-        gtk_container_add (GTK_CONTAINER (bubble->priv->iconbox), bubble->priv->icon);
+
+        gtk_box_pack_start (GTK_BOX (bubble->priv->main_hbox),
+                            bubble->priv->icon,
+                            FALSE, FALSE, 0);
+
+        /* Add vbox */
 
         vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
         gtk_widget_show (vbox);
@@ -489,14 +500,16 @@ nd_bubble_init (NdBubble *bubble)
         gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
 
         /* Add the close button */
-        alignment = gtk_alignment_new (0.5, 0, 0, 0);
-        gtk_widget_show (alignment);
-        gtk_box_pack_start (GTK_BOX (bubble->priv->main_hbox), alignment, FALSE, FALSE, 0);
 
         close_button = gtk_button_new ();
+        gtk_widget_set_valign (close_button, GTK_ALIGN_START);
         gtk_widget_show (close_button);
+
         bubble->priv->close_button = close_button;
-        gtk_container_add (GTK_CONTAINER (alignment), close_button);
+        gtk_box_pack_start (GTK_BOX (bubble->priv->main_hbox),
+                            bubble->priv->close_button,
+                            FALSE, FALSE, 0);
+
         gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
         gtk_container_set_border_width (GTK_CONTAINER (close_button), 0);
         g_signal_connect (G_OBJECT (close_button),
@@ -506,9 +519,9 @@ nd_bubble_init (NdBubble *bubble)
 
         atkobj = gtk_widget_get_accessible (close_button);
         atk_action_set_description (ATK_ACTION (atkobj), 0,
-                                    "Closes the notification.");
+                                    _("Closes the notification."));
         atk_object_set_name (atkobj, "");
-        atk_object_set_description (atkobj, "Closes the notification.");
+        atk_object_set_description (atkobj, _("Closes the notification."));
 
         image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
         gtk_widget_show (image);
@@ -524,7 +537,7 @@ nd_bubble_init (NdBubble *bubble)
         gtk_label_set_line_wrap_mode (GTK_LABEL (bubble->priv->summary_label), PANGO_WRAP_WORD_CHAR);
 
         atkobj = gtk_widget_get_accessible (bubble->priv->summary_label);
-        atk_object_set_description (atkobj, "Notification summary text.");
+        atk_object_set_description (atkobj, _("Notification summary text."));
 
         bubble->priv->content_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
         gtk_widget_show (bubble->priv->content_hbox);
@@ -548,15 +561,13 @@ nd_bubble_init (NdBubble *bubble)
                           bubble);
 
         atkobj = gtk_widget_get_accessible (bubble->priv->body_label);
-        atk_object_set_description (atkobj, "Notification body text.");
-
-        alignment = gtk_alignment_new (1, 0.5, 0, 0);
-        gtk_widget_show (alignment);
-        gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, TRUE, 0);
+        atk_object_set_description (atkobj, _("Notification summary text."));
 
         bubble->priv->actions_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+        gtk_widget_set_halign (bubble->priv->actions_box, GTK_ALIGN_END);
         gtk_widget_show (bubble->priv->actions_box);
-        gtk_container_add (GTK_CONTAINER (alignment), bubble->priv->actions_box);
+
+        gtk_box_pack_start (GTK_BOX (vbox), bubble->priv->actions_box, FALSE, TRUE, 0);
 }
 
 static void
@@ -701,13 +712,13 @@ set_notification_icon (NdBubble  *bubble,
                 int pixbuf_width = gdk_pixbuf_get_width (scaled);
 
                 gtk_widget_show (bubble->priv->icon);
-                gtk_widget_set_size_request (bubble->priv->iconbox,
+                gtk_widget_set_size_request (bubble->priv->icon,
                                              MAX (BODY_X_OFFSET, pixbuf_width), -1);
                 g_object_unref (scaled);
                 bubble->priv->have_icon = TRUE;
         } else {
                 gtk_widget_hide (bubble->priv->icon);
-                gtk_widget_set_size_request (bubble->priv->iconbox,
+                gtk_widget_set_size_request (bubble->priv->icon,
                                              BODY_X_OFFSET,
                                              -1);
                 bubble->priv->have_icon = FALSE;
@@ -743,16 +754,8 @@ add_notification_action (NdBubble       *bubble,
         char      *buf;
 
         if (!gtk_widget_get_visible (bubble->priv->actions_box)) {
-                GtkWidget *alignment;
-
                 gtk_widget_show (bubble->priv->actions_box);
                 update_content_hbox_visibility (bubble);
-
-                alignment = gtk_alignment_new (1, 0.5, 0, 0);
-                gtk_widget_show (alignment);
-                gtk_box_pack_end (GTK_BOX (bubble->priv->actions_box),
-                                  alignment,
-                                  FALSE, TRUE, 0);
         }
 
         button = gtk_button_new ();
